// --------  GMD-MIDI_CONTROLLER_LP-v1.21.ino - -------------
//
// v1.21  -- Leopoldi
// An meine Hardware angepasste Aktualisierung des
// Originalsketch von Götz Müller-Dürholts MIDI-Controller
// auf der Plattform: Arudino NANO  -- angepasst auf PCB-03 von LP
//
// ---------------------------------------------------------

// Byte für ControlChange-Befehl auf MIDI Kanal 1
byte controlChange = 176;
//  
//CNR=Poti-Nr.:                                  --Res.--   T1 T2 Us Us
byte controllerNummer[] = 
       {20,21,22,23,24,25,26,27, 28,29,30,71,75,99,99,99,  72,73,74,76};
//Byte: 0  1  2  3  4  5  6  7 | 8  9 10 11 12 13 14 15 |  16 17 18 19

// Array für 2 achtfach Multiplexer = 16 Werte
int potiWert[16];
byte controllerWert[16];
byte controllerWertAlt[16];

// Array für 2 Taster mit LED, die als Schalter funktionieren
byte taste[] =             {HIGH,HIGH};     
byte tasteAlt[] =          {HIGH,HIGH};     
boolean tasteGedrueckt[] = {true, true};    
byte tastePin[] =          {9, 11};        
byte tasteLEDPin[] =       {10, 12};        

// 3 Bits für den Zähler der Multiplexer
byte bit1 = 0;
byte bit2 = 0;
byte bit3 = 0;

int L = 13;  //Test-LED

void setup() {
  //Select-Pins 4051s
  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  
  //Sensoren
  pinMode(5, OUTPUT); //Trigger 1
  pinMode(6, INPUT);  //Echo 1
  pinMode(7, OUTPUT); //Trigger 2         
  pinMode(8, INPUT);  //Echo 2             
  
  //Taster mit LED
  pinMode( 9, INPUT_PULLUP);  // Taster 1          
  pinMode(10, OUTPUT); // Taster 1 LED      
  pinMode(11, INPUT_PULLUP);  // Taster 2          
  pinMode(12, OUTPUT); // Taster 2 LED      

  Serial.begin(9600);   //MIDI-Ausgabe mit 31250
}

void loop() {
  // Schleife fragt alle Potis an den 4051 Multiplexern ab
  for (byte i = 0; i <= 7; i++) {
    bit1 = bitRead(i,0);
    bit2 = bitRead(i,1);
    bit3 = bitRead(i,2);
  
    digitalWrite(2, bit1);
    digitalWrite(3, bit2);
    digitalWrite(4, bit3);
    
   potisAbfragen(i,A0);
   potisAbfragen(i+8,A1);
  }

  // Fragt die beiden Taster ab
  tasteAbfragen(0 ,controllerNummer[16]);
  tasteAbfragen(1 ,controllerNummer[17]);

  // Fragt die Ultraschall-Sensoren ab
  sensorAbfragen(5, 6, controllerNummer[18]);
  sensorAbfragen(7, 8, controllerNummer[19]);

}

void potisAbfragen(byte zaehler, int analogPin) {
  //Formel um ein Poti abzufragen und die Messwerte zu glätten
  potiWert[zaehler] = 0.2 * potiWert[zaehler] + 0.8 * analogRead(analogPin);
  controllerWert[zaehler] = map(potiWert[zaehler],0,1023,0,127);
  if (controllerWert[zaehler] != controllerWertAlt[zaehler]) {
    sendeMIDI(controlChange, controllerNummer[zaehler], controllerWert[zaehler]);
    controllerWertAlt[zaehler] = controllerWert[zaehler];
  }
}



void sensorAbfragen(byte triggerPin, byte echoPin, byte ctrlNummer) {
  //Messung auslösen
  digitalWrite(triggerPin, LOW);
  delayMicroseconds(2);
  digitalWrite(triggerPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(triggerPin, LOW);
  
  long sensorWert = pulseIn(echoPin, HIGH);

  if ( (sensorWert <= 2000) && (sensorWert >= 200)) {
    byte sensorMIDI = map(sensorWert,2000,200,0,127);
    sendeMIDI(controlChange, ctrlNummer, sensorMIDI);

   // Serial.write(ctrlNummer);  //Test
  }
}

void tasteAbfragen(byte x, byte ctrlNummer) {
  taste[x] = digitalRead(tastePin[x]);
                                        
  if (taste[x] == LOW && tasteAlt[x] == HIGH) {
    if(tasteGedrueckt[x] == false){
      sendeMIDI(controlChange, ctrlNummer, 0);  
      digitalWrite(tasteLEDPin[x], LOW);
      tasteGedrueckt[x] = true;
    }
    else {
      sendeMIDI(controlChange, ctrlNummer, 127);
      digitalWrite(tasteLEDPin[x], HIGH);
      tasteGedrueckt[x] = false;
    }
  }
  tasteAlt[x] = taste[x];
}


// Funktion zum Verschicken eines MIDI-Befehls
void sendeMIDI(byte statusByte, byte dataByte1, byte dataByte2) {
  Serial.write(statusByte);
  Serial.write(dataByte1);
  Serial.write(dataByte2);
}
